<?php

namespace Drupal\type_tray_icon_display\Plugin\Block;

use Drupal\Core\Access\AccessResultForbidden;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\Entity\NodeType;

/**
 * Provides a block to view a specific entity.
 *
 * @Block(
 *   id = "type_tray_icon_display_block",
 *   admin_label = @Translation("Type Tray Icon Display"),
 *   category = @Translation("Type Tray"),
 * )
 */
class IconBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account, $return_as_object = FALSE) {
    // Check the parent's access.
    $parent_access = parent::access($account, TRUE);
    if (!$parent_access->isAllowed()) {
      return $return_as_object ? $parent_access : $parent_access->isAllowed();
    }

    $entity = NULL;
    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $this->getEntity();

    if ($entity) {
      return $entity->access('view', $account, $return_as_object);
    }
    return new AccessResultForbidden("No Entity Found.");
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];

    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $this->getEntity();

    if (!$entity) {
      return;
    }
    // @TODO reduce duplication of code between here and our Extra Field display

    // For now, Type Tray only applies to nodes.  See the feature request
    // https://www.drupal.org/project/type_tray/issues/3438075
    if ($entity->getEntityTypeId() !== 'node') {
      return;
    }
    /** @var \Drupal\node\NodeInterface $entity */
    $content_type = $entity->getType();

    $content_type_config = NodeType::load($content_type);
    if (isset($content_type_config)) {
      $third_party_property = $content_type_config->get('third_party_settings');
      if (!empty($third_party_property['type_tray']['type_icon'])) {
        $url = $third_party_property['type_tray']['type_icon'];
        $build['icon'] = [
          '#type' => 'component',
          '#component' => 'type_tray_icon_display:icon',
          '#props' => [
            'url' => $url,
            'label' => $content_type_config->label(),
          ],
        ];
      }
    }

    return $build;
  }

  /**
  * {@inheritdoc}
  */
  public function getCacheMaxAge() {
    // We cache by tag (content type) not by time.
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    // Type Tray only supports nodes.
    /** @var \Drupal\node\NodeInterface $entity */
    $entity = $this->getEntity();
    // Rebuild cache if the step is a new one.
    if ($entity) {
      // @TODO invalidate our tag when content type settings forms are saved.
      return Cache::mergeTags(parent::getCacheTags(), ['node:' . $entity->getType()]);
    }
    else {
      return parent::getCacheTags();
    }
  }

  /**
   * Gets the current entity from the available context (the page URL).
   *
   * Currently Type Tray only supports nodes so that's all we get.
   */
  public function getEntity() {
    // This is ridiculous but a simple getParamater('node') works in access()
    // when we use the view_mode_page module, but not in the build() method.
    if (
      \Drupal::routeMatch()->getRouteName() === 'view_mode_page.display_entity'
      && \Drupal::routeMatch()->getParameter('entity_type') === 'node'
      && $nid = \Drupal::routeMatch()->getParameter('entity_id')
    ) {
      return \Drupal\node\Entity\Node::load($nid);
    }
    else {
      // \Drupal::routeMatch()->getRouteName() == 'entity.node.canonical'
      // … presumably, or not a match, but no error so we won't limit it, so
      // that we catch anything that'll give us a node.
      return \Drupal::routeMatch()->getParameter('node');
    }
  }

}
